#include "kiss.h"

/* Copyright (c) 2009, Zoe Stephenson.  All rights reserved, redistribution
 * permitted under the terms of the BSD license (see LICENSE for details) */

/*- Package description

  Kiss ADT functions:

    This is a thin ADT - allocation is catered for, but access
    and destruction are done directly.

*/

kiss_parameter *new_kiss_parameter(void)
{
    kiss_parameter
        *k=NEW(kiss_parameter);
    PROTECT(k)
    {
        k->environmentName=NULLstring;
        k->argumentName=NULLstring;
        k->defaultValue=NULLstring;
        k->currentValue=NULLstring;
        k->id=UndefinedParameter;
    }
    return k;
}

kiss_parameter_list *new_kiss_parameter_list(void)
{
    kiss_parameter_list
        *k=NEW(kiss_parameter_list);
    PROTECT(k)
    {
        k->content=NULLkiss_parameter;
        k->next=NULLkiss_parameter_list;
    }
    return k;
}

kiss_config *new_kiss_config(void)
{
    kiss_config
        *k=NEW(kiss_config);
    PROTECT(k)
    {
        k->kiss_display=NULLstring;
        k->kiss_client=NULLstring;
        k->kiss_screen=KISS_BAD_SCREEN;
        k->kiss_delay=(Time)0;
    }
    return k;
}

kiss_target *new_kiss_target(void)
{
    kiss_target
        *k=NEW(kiss_target);
    PROTECT(k)
    {
        k->kiss_Xdisplay=(Display *)0;
        k->kiss_RootWindow=-1;
        k->kiss_Xscreen=-1;
        k->minX=0;
        k->maxX=0;
        k->minY=0;
        k->maxY=0;
    }
    return k;
}

kiss *new_kiss(void)
{
    kiss
        *k=NEW(kiss);
    PROTECT(k)
    {
        k->x=0;
        k->y=0;
        k->state=0;
        k->time=(Time)0;
    }
    return k;
}

kiss_list *new_kiss_list(void)
{
    kiss_list
        *k=NEW(kiss_list);
    PROTECT(k)
    {
        k->content=NULLkiss;
        k->next=NULLkiss_list;
    }
    return k;
}

kiss_state *new_kiss_state(void)
{
    kiss_state
        *k=NEW(kiss_state);
    PROTECT(k)
    {
        side
            l;

        for(l=NoSide; l<MaxSide; l++)
        {
            k->location[l]=False;
        }
    }
    return k;
}

