/* Copyright (c) 2009, Zoe Stephenson.  All rights reserved, redistribution
 *  * permitted under the terms of the BSD license (see LICENSE for details) */

/*- Combined header file

 Kiss definitions:

   Some definitions are kept local to the package where they are
   used.  In this case, see the section 'Local definitions' in
   that package file.

*/

/*- Headers */

/* TODO: Check where each of these is actually used */

/* 
  functions
    exit, malloc, realloc, free, setenv
  constants
    EXIT_SUCCESS, EXIT_FAILURE
*/
#include <stdlib.h>

/*
  functions
    ferror, feof, fread, fwrite, fopen, freopen, fflush, printf, fprintf,
    sprintf
  types
    FILE
*/
#include <stdio.h>

/*
  functions
    XOpenDisplay, XCloseDisplay, XSelectInput, XNextEvent, DisplayWidth,
    DisplayHeight, DefaultScreen, ScreenCount, RootWindow
  constants
    True, False, ControlMask, LockMask, ShiftMask, Mod1Mask, Mod2Mask,
    Mod3Mask, Mod4Mask, Mod5Mask, Button1Mask, Button2Mask, Button3Mask,
    Button4Mask, Button5Mask
  types
    Time, XEvent, XPointerMovedEvent, Bool, Display
*/
#include <X11/Xlib.h>

/*
  functions
    execvp, fork, unlink
*/
#include <unistd.h>

/*
  types
    pid_t, fd_set
*/
#include <sys/types.h>

/*
  functions
    wait, WIFEXITED, WIFSIGNALED, WEXITSTATUS, WTERMSIG
*/
#include <sys/wait.h>

/*
  functions
    stat, mknod, S_ISDIR
  constants
    S_IFIFO
  types
    struct stat
*/
#include <sys/stat.h>

/*
  functions
    select, FD_ZERO, FD_SET
  types
    struct timeval
*/
#include <sys/time.h>

/*
  functions
    strcpy, strncmp, strcmp, strlen, strcat
*/
#include <string.h>

/*
  functions
    strerror
  values
    errno
*/
#include <errno.h>

/*
  functions
    kill
  constants
    SIGTERM, SIGCHLD, SIGUSR1, SIGUSR2
*/
#include <signal.h>

/*- Misc. */

/* A little debuggery for when it goes wrong */

#define DEBUG(X) printf("[Debug]:" X "\n")
#define DEBUG_INT(X) printf("[Debug]: %d\n", X)

/* I find it easier to read my code if I use these: */

#define NEW(X) ((X *)malloc(sizeof(X)))
#define PROTECT(X) if((any)(X) != NULLany)

/*- Defaults */

#define DEFAULT_KISS_DISPLAY ":0"
#define DEFAULT_KISS_SCREEN "!"
#define DEFAULT_KISS_CLIENT "/usr/local/bin/kiss-client"
#define DEFAULT_KISS_USER_CLIENT ".kiss-client"
#define DEFAULT_KISS_DELAY "500"
#define DEFAULT_KISS_FIFOS "/var/tmp"

#define KISS_BAD_SCREEN (-1)

/*- Types */

/* Xlib defines True and False.  Currently, they're set to 1
   and 0 respectively.  We want to add another constant, Undefined,
   to that - but without specifying its value.  I think the following
   does that:
   
   C false is always 0.  So, assume False will always be set to 0.
   Undefined must not be the same as True.  True will not be 0, as
   0 is False.  So, -(True) will be different from True.  So, use
   that as the Undefined value.  Plus, Bool is defined as an int,
   so the negation won't be a problem (unless True is the negative
   number with the greatest magnitude...) */

#define Undefined (-(True))
#define MakeBool(X) ((X)?(True):(False))

/* An alternative is to use the compiler itself:

  enum { DUMMYTrue=True, DUMMYFalse=False, Undefined };

*/

/* Parameter IDs to uniquely identify parameters */
typedef enum {UndefinedParameter=Undefined, DisplayParameter, ScreenParameter,
DelayParameter, ClientParameter, FifoParameter, MaxParameters} parameter_id;

/* The screen has four sides */
typedef enum {NoSide, LeftSide, RightSide, TopSide, BottomSide, MaxSide} side;

/* When we get a response from a routine with no other return value: */
typedef enum {UndefinedResponse=Undefined, SuccessResponse, FailureResponse, MaxResponse } response;

#define NULLstring ((char *)0)

/* If we're going to use (void *) to pass things of any type, then
   we'll call it any type. */

typedef void *any;

#define NULLany ((any)0)

/* Parameter arguments, environment variables and defaults to use */
typedef struct kiss_parameter
{
    char
        *environmentName,
        *argumentName,
        *defaultValue,
        *currentValue;
    parameter_id
        id;
} kiss_parameter;

#define NULLkiss_parameter ((kiss_parameter *)0)
kiss_parameter *new_kiss_parameter(void);

/* And lists of them */
struct kiss_parameter_list;
struct kiss_parameter_list
{
    kiss_parameter
        *content;
    struct kiss_parameter_list
        *next;
};
typedef struct kiss_parameter_list kiss_parameter_list;

#define NULLkiss_parameter_list ((kiss_parameter_list *)0)
kiss_parameter_list *new_kiss_parameter_list(void);

/* The current configuration of Kiss, as parsed from parameters */
typedef struct kiss_config
{
    char
        *kiss_fifos,
        *kiss_display,
        *kiss_client;
    int
        kiss_screen;
    /* TODO Check to see if we need this to be Time */
    Time
        kiss_delay;
} kiss_config;

#define NULLkiss_config ((kiss_config *)0)
kiss_config *new_kiss_config(void);

/* Information regarding the target to be monitored */
typedef struct kiss_target
{
    Display
        *kiss_Xdisplay;
    Window
        kiss_RootWindow;
    int
        kiss_Xscreen,
        minX,
        maxX,
        minY,
        maxY;
} kiss_target;
    
#define NULLkiss_target ((kiss_target *)0)
kiss_target *new_kiss_target(void);

/* Information about a single kiss */
typedef struct kiss
{
    int
        x,
        y;
    unsigned int
        state;
    Time
        time;
} kiss;

#define NULLkiss ((kiss *)0)
kiss *new_kiss(void);

/* A list of kisses */
struct kiss_list;
struct kiss_list
{
    kiss
        *content;
    struct kiss_list
        *next;
};
typedef struct kiss_list kiss_list;

#define NULLkiss_list ((kiss_list *)0)
kiss_list *new_kiss_list(void);

/* The state of the kiss detector at any one time */
typedef struct kiss_state
{
    Bool
        location[MaxSide];
} kiss_state;

#define NULLkiss_state ((kiss_state *)0)
kiss_state *new_kiss_state(void);

/* The name of a FIFO */
typedef char *kiss_fifo;
#define NULLkiss_fifo ((kiss_fifo)0)


/*- Prototypes */

/* kmonitor.c */
void kiss_monitor(kiss_fifo output, kiss_target *target);

/* ktrigger.c */
void kiss_trigger(kiss_fifo input, kiss_fifo output, kiss_config *config, kiss_target *target);

/* kclient.c */
response kiss_client_monitor(kiss_fifo client_input, kiss_config *config);
response signal_kiss_client(char *signaltext, FILE *s);

/* kparameters.c */
void add_kiss_parameter(kiss_parameter_list *params, parameter_id id, char *eName, char *aName, char *dValue);
void parse_parameters(kiss_parameter_list *params, kiss_config *config, int argc, char *argv[]);

/* kfifo.c */
kiss_fifo new_fifo(kiss_config *config);
void kill_fifo(kiss_fifo f);
void clean_all_fifos(void);

/* kdump.c */
void dump_kiss_list(kiss_list *queue);

/* kerror.c */
void error_init(char *programname);
void error_fatal(int errornumber, char *description);
void error_warning(int errornumber, char *description);

